#!/usr/bin/env bash

setup_file(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _add_server
  _new_app
}

teardown_file(){
  _rm_app
  _rm_server
}

setup(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
}

teardown(){
  # https://github.com/bats-core/bats-core/issues/383#issuecomment-738628888
  if [[ -z "${BATS_TEST_COMPLETED}" ]]; then
    _undeploy_app
  fi
}

@test "validate app argument" {
  run $ABRA app rollback
  assert_failure
  assert_output --partial 'no app provided'

  run $ABRA app rollback DOESNTEXIST
  assert_failure
  assert_output --partial 'cannot find app'
}

@test "retrieve recipe if missing" {
  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE"
  assert_success
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE"

  run $ABRA app rollback "$TEST_APP_DOMAIN" --no-input --no-converge-checks
  assert_failure
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE"
}

@test "ensure recipe up to date if no --offline" {
  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial 'behind 3'

  run $ABRA app rollback "$TEST_APP_DOMAIN" --no-input --no-converge-checks
  assert_failure

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  refute_output --partial 'behind 3'
}

@test "ensure recipe not up to date if --offline" {
  latestCommit="$(git -C "$ABRA_DIR/recipes/$TEST_RECIPE" rev-parse --short HEAD)"
  refute [ -z "$latestCommit" ];

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial 'behind 3'

  run $ABRA app rollback "$TEST_APP_DOMAIN" \
    --no-input --no-converge-checks --offline
  assert_failure

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial 'behind 3'

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" checkout "$latestCommit"
  assert_success

  _reset_recipe "$TEST_RECIPE"
}

@test "bail if unstaged changes and no --chaos" {
  run bash -c "echo foo >> $ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_success
  assert_output --partial 'foo'

  run $ABRA app rollback "$TEST_APP_DOMAIN" --no-input --no-converge-checks
  assert_failure
  assert_output --partial 'locally unstaged changes'
  refute_output --partial 'chaos'

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

# bats test_tags=slow
@test "do not bail if unstaged changes and --chaos" {
  run bash -c 'echo "unstaged changes" >> "$ABRA_DIR/recipes/$TEST_RECIPE/foo"'
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_success
  assert_output --partial 'foo'

  run $ABRA app deploy "$TEST_APP_DOMAIN" \
    --chaos --no-input --no-converge-checks
  assert_success
  assert_output --partial 'chaos'

  run $ABRA app rollback "$TEST_APP_DOMAIN" \
    --chaos --no-input --no-converge-checks
  assert_success
  assert_output --partial 'chaos'

  _undeploy_app

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

# bats test_tags=slow
@test "ensure same commit if --chaos" {
  latestCommit="$(git -C "$ABRA_DIR/recipes/$TEST_RECIPE" rev-parse --short HEAD)"

  run $ABRA app deploy "$TEST_APP_DOMAIN" \
    --no-input --no-converge-checks --chaos
  assert_success
  assert_output --partial "$latestCommit"
  assert_output --partial 'chaos'

  run $ABRA app rollback "$TEST_APP_DOMAIN" \
    --chaos --no-input --no-converge-checks
  assert_success
  assert_output --partial "$latestCommit"
  assert_output --partial 'chaos'

  _undeploy_app
}

# bats test_tags=slow
@test "no rollback if lint error" {
  _deploy_app

  run sed -i '/traefik.enable=.*/d' "$ABRA_DIR/recipes/$TEST_RECIPE/compose.yml"
  assert_success

  run $ABRA app rollback "$TEST_APP_DOMAIN" \
    --no-input --chaos --chaos --no-converge-checks
  assert_failure
  assert_output --partial 'failed lint checks'

  _undeploy_app

  _checkout_recipe "$TEST_RECIPE"
}

@test "error if not already deployed" {
  run $ABRA app rollback "$TEST_APP_DOMAIN" --no-input
  assert_failure
  assert_output --partial 'not deployed'
}

# bats test_tags=slow
@test "no rollback if on oldest version" {
  run $ABRA app deploy "$TEST_APP_DOMAIN" "0.1.0+1.20.0" --no-input --no-converge-checks
  assert_success
  assert_output --partial '0.1.0+1.20.0'

  run $ABRA app rollback "$TEST_APP_DOMAIN" --no-input --no-converge-checks
  assert_success
  assert_output --partial "you're on oldest"

  _undeploy_app
}

# bats test_tags=slow
@test "error if specific version is not downgrade" {
  run $ABRA app deploy "$TEST_APP_DOMAIN" "0.1.0+1.20.0" --no-input --no-converge-checks
  assert_success
  assert_output --partial '0.1.0+1.20.0'

  run $ABRA app rollback "$TEST_APP_DOMAIN" "0.1.0+1.20.0" --no-input --no-converge-checks
  assert_failure
  assert_output --partial 'is not a downgrade'

  _undeploy_app
}

# bats test_tags=slow
@test "rollback to previous version" {
  run $ABRA app deploy "$TEST_APP_DOMAIN" "0.1.1+1.20.2" --no-input --no-converge-checks
  assert_success
  assert_output --partial '0.1.1+1.20.2'

  run $ABRA app rollback "$TEST_APP_DOMAIN" --no-input --no-converge-checks
  assert_success
  assert_output --partial "0.1.0+1.20.0"

  _undeploy_app
}

# bats test_tags=slow
@test "rollback to a version 2 tags behind" {
  run $ABRA app deploy "$TEST_APP_DOMAIN" "0.2.0+1.21.0" --no-input --no-converge-checks
  assert_success
  assert_output --partial "0.2.0+1.21.0"

  run $ABRA app rollback "$TEST_APP_DOMAIN" "0.1.0+1.20.0" --no-input --no-converge-checks
  assert_success
  assert_output --partial "0.1.0+1.20.0"

  _undeploy_app
}
